<?php

declare(strict_types=1);

namespace Drupal\simple_oauth_redirect_uri_wildcard\Entity;

use Drupal\consumers\Entity\Consumer;
use Drupal\Core\Url;
use Drupal\simple_oauth\Entities\ClientEntity as ClientEntityBase;
use Drupal\simple_oauth_redirect_uri_wildcard\KeyValueStore\AuthCodeKeyValueStore;
use Drupal\simple_oauth_redirect_uri_wildcard\Validator\RedirectUriValidator;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Extends the base client entity but set custom redirect uris.
 */
class ClientEntity extends ClientEntityBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    Consumer $entity,
    protected readonly Request $request,
    protected readonly AuthCodeKeyValueStore $authCodeKv,
    protected readonly LoggerInterface $logger,
    protected readonly array $redirectUris,
  ) {
    parent::__construct($entity);
  }

  /**
   * Allow override of configured redirect uris.
   *
   * {@inheritdoc}
   */
  public function setRedirectUri(Consumer $entity): void {
    parent::setRedirectUri($entity);

    $redirectUri = $this->getRedirectUriFromRequest();

    if ($redirectUri) {
      $this->addRedirectUriOnDynamicMatch($redirectUri);
    }
  }

  /**
   * Get the current redirect_uri from request.
   */
  protected function getRedirectUriFromRequest(): ?string {
    $request = $this->request;

    // Redirect uri is in current request query params.
    if ($redirectUri = $request->query->get('redirect_uri')) {
      return $redirectUri;
    }

    // On /oauth/token route, get from request payload.
    if ($request->getPathInfo() === Url::fromRoute('oauth2_token.token')->toString()) {
      $data = [];
      parse_str($request->getContent(), $data);

      // Wrong payload.
      if (!array_key_exists('code', $data)) {
        return NULL;
      }

      return $this->authCodeKv->get($data['code']);
    }

    return NULL;
  }

  /**
   * Add dynamic redirect uri from query.
   *
   * If the given redirect uri matches any redirect uri
   * configured in the client (including wildcard matches),
   * add it to the redirect uris.
   */
  protected function addRedirectUriOnDynamicMatch(string $redirectUri): void {
    $validator = new RedirectUriValidator();

    foreach ($this->redirectUris as $schema) {
      if ($validator->uriMatchesSchema($redirectUri, $schema)) {
        // Log if allowed by wildcard.
        if (strpos($schema, '*') !== FALSE) {
          $this->logger->debug('Allowing dynamic redirect uri "@redirectUri" via wildcard "@schema".', [
            '@redirectUri' => $redirectUri,
            '@schema' => $schema,
          ]);
        }

        $this->redirectUri[] = $redirectUri;
      }
    }
  }

}
