<?php

declare(strict_types=1);

namespace Drupal\Tests\simple_oauth_redirect_uri_wildcard\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\consumers\Entity\Consumer;

/**
 * Tests the wildcard redirect URI functionality.
 *
 * @group simple_oauth_redirect_uri_wildcard
 */
class WildcardRedirectUriBrowserTest extends BrowserTestBase {

  protected const ERROR_BODY = '{"error":"invalid_client","error_description":"Client authentication failed"}';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'simple_oauth',
    'simple_oauth_redirect_uri_wildcard',
    'consumers',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A test user with permissions to manage OAuth2 clients.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->adminUser = $this->drupalCreateUser([
      'administer consumer entities',
      'administer simple_oauth entities',
      'administer oauth2 scopes',
      'access content',
    ]);

    // Create test keys.
    $this->generateKeys();
  }

  /**
   * Generate testing keys.
   */
  protected function generateKeys() {
    $key_dir = $this->publicFilesDirectory . '/keys';
    if (!file_exists($key_dir)) {
      mkdir($key_dir, 0777, TRUE);
    }

    $private_key = openssl_pkey_new();
    openssl_pkey_export_to_file($private_key, $key_dir . '/private.key');
    $public_key = openssl_pkey_get_details($private_key)['key'];
    file_put_contents($key_dir . '/public.key', $public_key);

    $this->config('simple_oauth.settings')
      ->set('public_key', $key_dir . '/public.key')
      ->set('private_key', $key_dir . '/private.key')
      ->save();
  }

  /**
   * Tests the wildcard redirect URI functionality.
   */
  public function testWildcardRedirectUri() {
    // Create a client with wildcard redirect URI.
    $client = Consumer::create([
      'label' => 'Test Client',
      'client_id' => 'test_client',
      'secret' => 'test_secret',
      'confidential' => TRUE,
      'redirect' => [
        ['value' => 'https://drupal.org/callback'],
        ['value' => 'https://w*w.drupal.org/callback'],
      ],
      'grant_types' => [
        ['value' => 'authorization_code'],
      ],
      'automatic_authorization' => FALSE,
      'remember_approval' => FALSE,
    ]);
    $client->save();

    // Test valid redirect URIs.
    $valid_redirects = [
      'https://drupal.org/callback',
      'https://www.drupal.org/callback',
    ];

    foreach ($valid_redirects as $redirect) {
      $this->drupalGet('oauth/authorize', [
        'query' => [
          'response_type' => 'code',
          'client_id' => 'test_client',
          'redirect_uri' => $redirect,
          'scope' => 'basic',
          'state' => 'test_state',
        ],
      ]);
      $this->assertSession()->pageTextNotContains(self::ERROR_BODY);
    }

    // Test invalid redirect URIs.
    $invalid_redirects = [
      'https://malicious.com/callback',
      'https://wrong-sub.drupal.org/callback',
      'http://drupal.org/callback',
    ];

    foreach ($invalid_redirects as $redirect) {
      $this->drupalGet('oauth/authorize', [
        'query' => [
          'response_type' => 'code',
          'client_id' => 'test_client',
          'redirect_uri' => $redirect,
          'scope' => 'basic',
          'state' => 'test_state',
        ],
      ]);
      $this->assertSession()->pageTextContains(self::ERROR_BODY);
    }

    // Set redirect uri to https://*.drupal.org/callback
    // Now wrong-sub must work.
    $client->set('redirect', [
      ['value' => 'https://*.drupal.org/callback'],
    ])->save();

    $this->drupalGet('oauth/authorize', [
      'query' => [
        'response_type' => 'code',
        'client_id' => 'test_client',
        'redirect_uri' => 'https://wrong-sub.drupal.org/callback',
        'scope' => 'basic',
        'state' => 'test_state',
      ],
    ]);
    $this->assertSession()->pageTextNotContains(self::ERROR_BODY);
  }

  /**
   * Tests adding a redirect URI with wildcard through the consumer form.
   */
  public function testAddWildcardRedirectUriInForm(): void {
    $this->drupalLogin($this->adminUser);

    // Create a new consumer through the form.
    $this->drupalGet('admin/config/services/consumer/add');
    $this->assertSession()->statusCodeEquals(200);

    $edit = [
      'label[0][value]' => 'Test Wildcard Client',
      'client_id[0][value]' => 'test_wildcard_client',
      'new_secret' => 'test_secret',
      'confidential[value]' => TRUE,
      'redirect[0][value]' => 'http://te*.mydrupal.local:3000/auth/callback',
      'grant_types[authorization_code]' => TRUE,
    ];
    $this->submitForm($edit, 'Save');

    // Assert the consumer was created successfully.
    $this->assertSession()->pageTextContains('Created the Test Wildcard Client Consumer.');

    // Load the created consumer and verify the redirect
    // URI was saved correctly.
    $client = Consumer::load(2);
    $this->assertEquals('http://te*.mydrupal.local:3000/auth/callback', $client->get('redirect')->getValue()[0]['value']);
  }

}
